package eu.mhsl.craftattack.spawn.appliances.privateMessage;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.chatMessages.ChatMessages;
import eu.mhsl.craftattack.spawn.appliances.privateMessage.commands.PrivateMessageCommand;
import eu.mhsl.craftattack.spawn.appliances.privateMessage.commands.PrivateReplyCommand;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

import java.util.*;
import java.util.stream.Collectors;

public class PrivateMessage extends Appliance {
    public final int targetChangeTimeoutSeconds = 30;

    private record Conversation(UUID target, Long lastSet) {}
    private final Map<Player, List<Conversation>> replyMapping = new WeakHashMap<>();

    public void reply(Player sender, String message) {
        if(this.replyMapping.get(sender) == null || this.replyMapping.get(sender).isEmpty()) throw new ApplianceCommand.Error("Du kannst nicht auf eine Konversation antworten, da keine vorhanden ist.");

        Conversation youngestEntry = this.replyMapping.get(sender).stream()
            .max(Comparator.comparingLong(o -> o.lastSet))
            .orElse(this.replyMapping.get(sender).getLast());

        if(youngestEntry.lastSet < System.currentTimeMillis() - (targetChangeTimeoutSeconds*1000) || this.replyMapping.get(sender).size() == 1) {
            Player target = Bukkit.getPlayer(youngestEntry.target());
            if(target == null) throw new ApplianceCommand.Error("Der Spieler " + Bukkit.getOfflinePlayer(youngestEntry.target()).getName() + " ist nicht mehr verfügbar.");

            this.replyMapping.get(sender).clear();
            sendWhisper(sender, new ResolvedPmUserArguments(target, message));
            return;
        }

        List<Conversation> oldConversations = this.replyMapping.get(sender).stream()
            .filter(conversation -> conversation.lastSet < System.currentTimeMillis() - (targetChangeTimeoutSeconds*1000))
            .toList();
        this.replyMapping.get(sender).removeAll(oldConversations);

        List<String> playerNames = this.replyMapping.get(sender).stream()
            .map(conversation -> {
                if(Bukkit.getPlayer(conversation.target()) != null) {
                    return Objects.requireNonNull(Bukkit.getPlayer(conversation.target())).getName();
                } else {
                    return Bukkit.getOfflinePlayer(conversation.target()).getName();
                }
            })
            .toList();

        sender.sendMessage(
            Component.text()
                .append(Component.text("Das Ziel für /r hat sich bei dir in den letzten "))
                .append(Component.text(String.valueOf(this.targetChangeTimeoutSeconds)))
                .append(Component.text(" Sekunden geändert. Wer soll deine Nachricht bekommen? "))
        );

        
        Component newTextComponent = Component.text("");

        String firstTargetName;
        if(Bukkit.getPlayer(youngestEntry.target()) == null) {
            firstTargetName = Bukkit.getOfflinePlayer(youngestEntry.target()).getName();
        } else {
            firstTargetName = Objects.requireNonNull(Bukkit.getPlayer(youngestEntry.target())).getName();
        }

        if(firstTargetName != null) {
            Component nameComponent = Component.text("");
            nameComponent.append(Component.text(firstTargetName, NamedTextColor.LIGHT_PURPLE));

            newTextComponent
                    .append(nameComponent.clickEvent(ClickEvent.runCommand("/msg " + firstTargetName + " " + message)));
            newTextComponent.append(Component.text(" "));
        }


        playerNames.forEach(playerName -> {
            Component nameComponent = Component.text(playerName, NamedTextColor.LIGHT_PURPLE);
            newTextComponent
                    .append(nameComponent.clickEvent(ClickEvent.runCommand("/msg " + playerName + " " + message)));
            newTextComponent.append(Component.text(" "));
        });

        sender.sendMessage(newTextComponent);

    }

    public void sendWhisper(Player sender, ResolvedPmUserArguments userArguments) {

        // Ältere Einträge bei replyMapping für den receiver entfernen
        if(!(this.replyMapping.get(userArguments.receiver) == null)) {
            List<Conversation> oldEntries = this.replyMapping.get(userArguments.receiver).stream()
                .filter(conversation -> conversation.target() == sender.getUniqueId())
                .toList();
            this.replyMapping.get(userArguments.receiver).removeAll(oldEntries);
        }

        // Neuen Eintrag bei replyMapping für den receiver
        Conversation newReceiverConversation = new Conversation(
            sender.getUniqueId(),
            System.currentTimeMillis()
        );

        if(this.replyMapping.get(userArguments.receiver) == null) {
            List<Conversation> receiverConversationList = new ArrayList<>();
            receiverConversationList.add(newReceiverConversation);

            this.replyMapping.put(
                userArguments.receiver,
                receiverConversationList
            );
        } else {
            this.replyMapping.get(userArguments.receiver).add(newReceiverConversation);
        }

        // Einträge für sender bei replyMapping überschreiben mit receiver
        List<Conversation> senderConversationList = new ArrayList<>();
        senderConversationList.add(
            new Conversation(
                userArguments.receiver.getUniqueId(),
                System.currentTimeMillis()
            )
        );

        this.replyMapping.put(
            sender,
            senderConversationList
        );


        ChatMessages chatMessages = Main.instance().getAppliance(ChatMessages.class);
        Component privatePrefix = Component.text("[Privat] ", NamedTextColor.LIGHT_PURPLE);

        sender.sendMessage(
            Component.text()
                .append(privatePrefix.clickEvent(ClickEvent.suggestCommand(String.format("/msg %s ", userArguments.receiver.getName()))))
                .append(sender.displayName())
                .append(Component.text(" zu ", NamedTextColor.GRAY))
                .append(chatMessages.getReportablePlayerName(userArguments.receiver))
                .append(Component.text(" > ", NamedTextColor.GRAY))
                .append(Component.text(userArguments.message))
        );
        userArguments.receiver.sendMessage(
            Component.text()
                .append(privatePrefix.clickEvent(ClickEvent.suggestCommand(String.format("/msg %s ", sender.getName()))))
                .append(chatMessages.getReportablePlayerName(sender))
                .append(Component.text(" zu ", NamedTextColor.GRAY))
                .append(userArguments.receiver.displayName())
                .append(Component.text(" > ", NamedTextColor.GRAY))
                .append(Component.text(userArguments.message))
        );
    }

    public record ResolvedPmUserArguments(Player receiver, String message) {}
    public ResolvedPmUserArguments resolveImplicit(String[] args) {
        if(args.length < 2) throw new ApplianceCommand.Error("Es muss ein Spieler sowie eine Nachricht angegeben werden.");
        List<String> arguments = List.of(args);
        Player targetPlayer = Bukkit.getPlayer(arguments.getFirst());
        if(targetPlayer == null) throw new ApplianceCommand.Error(String.format("Der Spieler %s konnte nicht gefunden werden.", arguments.getFirst()));
        String message = arguments.stream().skip(1).collect(Collectors.joining(" "));
        return new ResolvedPmUserArguments(targetPlayer, message);
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(
            new PrivateMessageCommand(),
            new PrivateReplyCommand()
        );
    }
}
